Imports System
Imports System.Collections

Namespace istia.st.articles.dao

	Public Class ArticlesDaoArrayList
		Implements istia.st.articles.dao.IArticlesDao

		Private articles As New ArrayList
		Private Const nbArticles As Integer = 4

		' constructeur par dfaut
		Public Sub New()
			' on construit qqs articles
			For i As Integer = 1 To nbArticles
				articles.Add(New Article(i, "article" + i.ToString, i * 10, i * 10, i * 10))
			Next
		End Sub

		' liste de tous les articles
		Public Function getAllArticles() As IList Implements IArticlesDao.getAllArticles
			' on retourne la liste des articles
			SyncLock Me
				Return articles
			End SyncLock
		End Function

		' suppression de tous les articles
		Public Sub clearAllArticles() Implements IArticlesDao.clearAllArticles
			' on vide la liste des articles
			SyncLock Me
				articles.Clear()
			End SyncLock
		End Sub

		' obtenir un article identifi par sa cl
		Public Function getArticleById(ByVal idArticle As Integer) As Article Implements IArticlesDao.getArticleById
			' on recherche l'article dans la liste
			SyncLock Me
				Dim ipos As Integer = posArticle(articles, idArticle)
				If ipos <> -1 Then
					Return CType(articles(ipos), Article)
				Else
					Return Nothing
				End If
			End SyncLock
		End Function

		' ajouter un article  la liste des articles
		Public Function ajouteArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.ajouteArticle
			' on ajoute l'article  la liste des articles
			SyncLock Me
				' on vrifie qu'il n'existe pas dj
				Dim ipos As Integer = posArticle(articles, unArticle.id)
				If ipos <> -1 Then
					Throw New Exception("L'article d'id [" + unArticle.id.ToString + "] existe dj")
				End If
				' on ajoute l'article
				articles.Add(unArticle)
				' on rend le rsultat
				Return 1
			End SyncLock
		End Function

		' modifier un article
		Public Function modifieArticle(ByVal articleNouveau As Article) As Integer Implements IArticlesDao.modifieArticle
			' on modifie un article
			SyncLock Me
				' on vrifie qu'il existe
				Dim ipos As Integer = posArticle(articles, articleNouveau.id)
				' le cas o il n'existe pas
				If ipos = -1 Then Return 0
				' il existe - on le modifie
				articles(ipos) = articleNouveau
				' on rend le rsultat
				Return 1
			End SyncLock
		End Function

		' supprimer un article identifi par sa cl
		Public Function supprimeArticle(ByVal idArticle As Integer) As Integer Implements IArticlesDao.supprimeArticle
			' suppression d'un article
			SyncLock Me
				' on vrifie qu'il existe
				Dim ipos As Integer = posArticle(articles, idArticle)
				' le cas o il n'existe pas
				If ipos = -1 Then Return 0
				' il existe - on le supprime
				articles.RemoveAt(ipos)
				' on rend le rsultat
				Return 1
			End SyncLock
		End Function

		' changer le stock d'un article identifi par sa cl
		Public Function changerStockArticle(ByVal idArticle As Integer, ByVal mouvement As Integer) As Integer Implements IArticlesDao.changerStockArticle
			' changer le stock d'un article
			SyncLock Me
				' on vrifie qu'il existe
				Dim ipos As Integer = posArticle(articles, idArticle)
				' le cas o il n'existe pas
				If ipos = -1 Then Return 0
				' il existe - on modifie son stock si on peut
				Dim unArticle As Article = CType(articles(ipos), Article)
				' on ne change le stock que s'il est suffisant
				If unArticle.stockactuel + mouvement >= 0 Then
					unArticle.stockactuel += mouvement
					Return 1
				Else
					Return 0
				End If
			End SyncLock
		End Function

		' chercher un article identifi par sa cl
		Private Function posArticle(ByVal listArticles As ArrayList, ByVal idArticle As Integer) As Integer
			' rend la position de l'article [idArticle] dans la liste ou -1 si pas trouv
			Dim unArticle As Article
			For i As Integer = 0 To listArticles.Count - 1
				unArticle = CType(listArticles(i), Article)
				If unArticle.id = idArticle Then
					Return i
				End If
			Next
			' pas trouv
			Return -1
		End Function
	End Class

End Namespace