Imports System
Imports System.Collections
Imports NUnit.Framework
Imports istia.st.articles.dao
Imports System.Threading
Imports Spring.Objects.Factory.Xml
Imports System.IO

Namespace istia.st.articles.tests

	<TestFixture()> _
	Public Class NunitTestArticlesArrayList
		' l'objet  tester
		Private articlesDao As IArticlesDao

		<SetUp()> _
		Public Sub init()
			' on rcupre une instance du fabricant d'objets Spring
			Dim factory As XmlObjectFactory = New XmlObjectFactory(New FileStream("spring-config.xml", FileMode.Open))
			' on demande l'instanciation de l'objet articles dao
			articlesDao = CType(factory.GetObject("articlesdao"), IArticlesDao)
		End Sub

		<Test()> _
		Public Sub testGetAllArticles()
			' vrification visuelle
			listArticles()
		End Sub

		<Test()> _
		Public Sub testClearAllArticles()
			' on supprime tous les articles
			articlesDao.clearAllArticles()
			' on demande tous les articles
			Dim articles As IList = articlesDao.getAllArticles
			' vrification : il doit y en avoir 0
			Assert.AreEqual(0, articles.Count)
		End Sub

		<Test()> _
		Public Sub testAjouteArticle()
			' suppression de tous les articles
			articlesDao.clearAllArticles()
			' vrification : la table des articles doit tre vide
			Dim articles As IList = articlesDao.getAllArticles
			Assert.AreEqual(0, articles.Count)
			' on ajoute deux articles
			articlesDao.ajouteArticle(New Article(3, "article3", 30, 30, 3))
			articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			' vrification : il doit y avoir deux articles
			articles = articlesDao.getAllArticles
			Assert.AreEqual(2, articles.Count)
			' vrification visuelle
			listArticles()
		End Sub

		<Test()> _
		Public Sub testSupprimeArticle()
			' suppression de tous les articles
			articlesDao.clearAllArticles()
			' vrification : la table des articles doit tre vide
			Dim articles As IList = articlesDao.getAllArticles
			Assert.AreEqual(0, articles.Count)
			' on ajoute deux articles
			articlesDao.ajouteArticle(New Article(3, "article3", 30, 30, 3))
			articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			' vrification : il doit y avoir 2 articles
			articles = articlesDao.getAllArticles
			Assert.AreEqual(2, articles.Count)
			' on supprime l'article 4
			articlesDao.supprimeArticle(4)
			' vrification : il doit rester 1 article
			articles = articlesDao.getAllArticles
			Assert.AreEqual(1, articles.Count)
			' vrification visuelle
			listArticles()
		End Sub

		<Test()> _
		Public Sub testModifieArticle()
			' suppression de tous les articles
			articlesDao.clearAllArticles()
			' vrification
			Dim articles As IList = articlesDao.getAllArticles
			Assert.AreEqual(0, articles.Count)
			' ajout de 2 articles
			articlesDao.ajouteArticle(New Article(3, "article3", 30, 30, 3))
			articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			' vrification
			articles = articlesDao.getAllArticles
			Assert.AreEqual(2, articles.Count)
			' recherche de l'article 3
			Dim unArticle As Article = articlesDao.getArticleById(3)
			' vrification
			Assert.AreEqual(unArticle.nom, "article3")
			' recherche article 4
			unArticle = articlesDao.getArticleById(4)
			' vrification
			Assert.AreEqual(unArticle.nom, "article4")
			' modification article 4
			articlesDao.modifieArticle(New Article(4, "article4", 44, 44, 44))
			' vrification
			unArticle = articlesDao.getArticleById(4)
			Assert.AreEqual(unArticle.prix, 44, 0.000001)
			' vrification visuelle
			listArticles()
		End Sub

		<Test()> _
		Public Sub testGetArticleById()
			' suppression des articles
			articlesDao.clearAllArticles()
			' vrification
			Dim articles As IList = articlesDao.getAllArticles
			Assert.AreEqual(0, articles.Count)
			' ajout de 2 articles
			articlesDao.ajouteArticle(New Article(3, "article3", 30, 30, 3))
			articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			' vrification
			articles = articlesDao.getAllArticles
			Assert.AreEqual(2, articles.Count)
			' recherche article 3
			Dim unArticle As Article = articlesDao.getArticleById(3)
			' vrification
			Assert.AreEqual(unArticle.nom, "article3")
			' recherche article 4
			unArticle = articlesDao.getArticleById(4)
			' vrification
			Assert.AreEqual(unArticle.nom, "article4")
		End Sub

		' listing cran
		Private Sub listArticles()
			Dim articles As IList = articlesDao.getAllArticles
			For i As Integer = 0 To articles.Count - 1
				Console.WriteLine(CType(articles(i), Article).ToString)
			Next
		End Sub

		<Test()> _
		Public Sub testArticleAbsent()
			' suppression de tous les articles
			articlesDao.clearAllArticles()
			' recherche article 1
			Dim article As article = articlesDao.getArticleById(1)
			' vrification
			Assert.IsNull(article)
			' modification d'un article inexistant
			Dim i As Integer = articlesDao.modifieArticle(New article(1, "1", 1, 1, 1))
			' a du modifier aucune ligne
			Assert.AreEqual(i, 0)
			' suppression article inexistant
			i = articlesDao.supprimeArticle(1)
			' a du supprimer aucune ligne
			Assert.AreEqual(0, i)
		End Sub

		<Test()> _
		Public Sub testChangerStockArticle()
			' suppression tous les articles
			articlesDao.clearAllArticles()
			' ajout d'un article
			Dim nbArticles As Integer = articlesDao.ajouteArticle(New Article(3, "article3", 30, 101, 3))
			Assert.AreEqual(nbArticles, 1)
			' ajout d'un article
			nbArticles = articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			Assert.AreEqual(nbArticles, 1)
			' cration de 100 threads
			Dim taches(99) As Thread
			For i As Integer = 0 To taches.Length - 1
				' on cre le thread i
				taches(i) = New Thread(New ThreadStart(AddressOf dcrmente))
				' on fixe le nom du thread
				taches(i).Name = "tache_" & i
				' on lance l'excution du thread i
				taches(i).Start()
			Next
			' on attend la fin de tous les threads
			For i As Integer = 0 To taches.Length - 1
				taches(i).Join()
			Next
			' vrifications - article 3 doit avoir un stock de 1
			Dim unArticle As Article = articlesDao.getArticleById(3)
			Assert.AreEqual(unArticle.nom, "article3")
			Assert.AreEqual(1, unArticle.stockactuel)
			' on dcrmente le stock de l'article 4
			Dim erreur As Boolean = False
			Dim nbLignes As Integer = articlesDao.changerStockArticle(4, -100)
			' vrification : son stock n'a pas du changer
			Assert.AreEqual(0, nbLignes)
			' vrification visuelle
			listArticles()
		End Sub

		Public Sub dcrmente()
			' thread lanc
			System.Console.Out.WriteLine(Thread.CurrentThread.Name + " lanc")
			' thread dcrmente le stock
			articlesDao.changerStockArticle(3, -1)
			' thread termin
			System.Console.Out.WriteLine(Thread.CurrentThread.Name + " termin")
		End Sub

	End Class
End Namespace
