Imports NUnit.Framework
Imports istia.st.articles.dao
Imports istia.st.articles.domain
Imports Spring.Objects.Factory.Xml
Imports System.IO

Namespace istia.st.articles.tests

	<TestFixture()> _
	Public Class NunitTestArticlesDomain

		' l'objet  tester
		Private articlesDomain As IArticlesDomain
		Private articlesDao As IArticlesDao

		<SetUp()> _
		Public Sub init()
			' on rcupre une instance du fabricant d'objets Spring
			Dim factory As XmlObjectFactory = New XmlObjectFactory(New FileStream("spring-config.xml", FileMode.Open))
			' on demande l'instanciation de l'objet articles dao
			articlesDao = CType(factory.GetObject("articlesdao"), IArticlesDao)
			' puis celui de l'aobjet articlesdomain
			articlesDomain = CType(factory.GetObject("articlesdomain"), IArticlesDomain)
		End Sub

		<Test()> _
		Public Sub getAllArticles()
			' vrification visuelle
			listArticles()
		End Sub

		<Test()> _
		Public Sub getArticleById()
			' suppression des articles
			articlesDao.clearAllArticles()
			' vrification
			Dim articles As IList = articlesDomain.getAllArticles
			Assert.AreEqual(0, articles.Count)
			' ajout de 2 articles
			articlesDao.ajouteArticle(New Article(3, "article3", 30, 30, 3))
			articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			' vrification
			articles = articlesDomain.getAllArticles
			Assert.AreEqual(2, articles.Count)
			' recherche article 3
			Dim unArticle As Article = articlesDomain.getArticleById(3)
			' vrification
			Assert.AreEqual(unArticle.nom, "article3")
			' recherche article 4
			unArticle = articlesDao.getArticleById(4)
			' vrification
			Assert.AreEqual(unArticle.nom, "article4")
		End Sub

		<Test()> _
		Public Sub acheterPanier()
			' suppression des articles
			articlesDao.clearAllArticles()
			' vrification
			Dim articles As IList = articlesDomain.getAllArticles
			Assert.AreEqual(0, articles.Count)
			' ajout de 2 articles
			articlesDao.ajouteArticle(New Article(3, "article3", 30, 30, 3))
			articlesDao.ajouteArticle(New Article(4, "article4", 40, 40, 4))
			' vrification
			articles = articlesDomain.getAllArticles
			Assert.AreEqual(2, articles.Count)
			' cration d'un panier avec deux achats
			Dim panier As New panier
			panier.ajouter(New Achat(New Article(3, "article3", 30, 30, 3), 10))
			panier.ajouter(New Achat(New Article(4, "article4", 40, 40, 4), 10))
			' vrifications
			Assert.AreEqual(700, panier.totalPanier, 0.000001)
			Assert.AreEqual(2, panier.achats.Count)
			' validation panier
			articlesDomain.acheter(panier)
			' vrifications
			Assert.AreEqual(0, articlesDomain.erreurs.Count)
			Assert.AreEqual(0, panier.achats.Count)
			' recherche article 3
			Dim unArticle As Article = articlesDomain.getArticleById(3)
			' vrification
			Assert.AreEqual(unArticle.stockactuel, 20)
			' recherche article 4
			unArticle = articlesDao.getArticleById(4)
			' vrification
			Assert.AreEqual(unArticle.stockactuel, 30)
			' nouveau panier
			panier.ajouter(New Achat(New Article(3, "article3", 30, 30, 3), 100))
			' validation panier
			articlesDomain.acheter(panier)
			' vrifications
			Assert.AreEqual(1, articlesDomain.erreurs.Count)
			' recherche article 3
			unArticle = articlesDomain.getArticleById(3)
			' vrification
			Assert.AreEqual(unArticle.stockactuel, 20)
		End Sub

		<Test()> _
		Public Sub testRetirerAchats()
			' suppression du contenu de ARTICLES
			articlesDao.clearAllArticles()
			' lit la table ARTICLES
			Dim articles As IList = articlesDao.getAllArticles()
			Assert.AreEqual(0, articles.Count)
			' insertion
			Dim article3 As New Article(3, "article3", 30, 30, 3)
			articlesDao.ajouteArticle(article3)
			Dim article4 As New Article(4, "article4", 40, 40, 4)
			articlesDao.ajouteArticle(article4)
			' lit la table ARTICLES
			articles = articlesDomain.getAllArticles()
			Assert.AreEqual(2, articles.Count)
			' cration d'un panier avec deux achats
			Dim monPanier As New Panier
			monPanier.ajouter(New Achat(article3, 10))
			monPanier.ajouter(New Achat(article4, 10))
			' vrifications
			Assert.AreEqual(700.0, monPanier.totalPanier, 0.000001)
			Assert.AreEqual(2, monPanier.achats.Count)
			' ajouter un article dj achet
			monPanier.ajouter(New Achat(article3, 10))
			' vrifications
			' le total doit tre pass  1000
			Assert.AreEqual(1000.0, monPanier.totalPanier, 0.000001)
			' toujours 2 articles dans le panier
			Assert.AreEqual(2, monPanier.achats.Count)
			' qt article 3 a du passer  20
			Dim unAchat As Achat = CType(monPanier.achats(0), Achat)
			Assert.AreEqual(20, unAchat.qte)
			' on retire l'article 3 du panier
			monPanier.enlever(3)
			' vrifications
			' le total doit tre pass  400
			Assert.AreEqual(400.0, monPanier.totalPanier, 0.000001)
			' 1 seul article dans le panier
			Assert.AreEqual(1, monPanier.achats.Count)
			' ce doit tre l'article n 4
			Assert.AreEqual(4, CType(monPanier.achats(0), Achat).article.id)
		End Sub

		' listing cran
		Private Sub listArticles()
			Dim articles As IList = articlesDomain.getAllArticles
			For i As Integer = 0 To articles.Count - 1
				Console.WriteLine(CType(articles(i), Article).ToString)
			Next
		End Sub

	End Class


End Namespace
