Imports System
Imports System.Collections
Imports System.Data.Odbc

Namespace istia.st.articles.dao

	Public Class ArticlesDaoPlainODBC
		Implements istia.st.articles.dao.IArticlesDao

		' champs privs
		Private connexion As OdbcConnection = Nothing
		Private DSN As String
		Private insertCommand As OdbcCommand
		Private updatecommand As OdbcCommand
		Private deleteSomeCommand As OdbcCommand
		Private selectSomeCommand As OdbcCommand
		Private updateStockCommand As OdbcCommand
		Private deleteAllCommand As OdbcCommand
		Private selectAllCommand As OdbcCommand

		' constructeur
		Public Sub New(ByVal DSN As String, ByVal uid As String, ByVal password As String)
			' DSN : nom de la source ODBC
			' uid : identit de l'utilisateur
			' password : son mot de passe

			'on rcupre le nom de la base pass en argument
			Me.DSN = DSN
			Dim connectString As String = String.Format("DSN={0};UID={1};PASSWORD={2}", DSN, uid, password)
			'on instancie la connexion
			connexion = New OdbcConnection(connectString)
			' on prpare les requtes SQL
			insertCommand = New OdbcCommand("insert into ARTICLES(id, nom, prix, stockactuel, stockminimum) values (?,?,?,?,?)", connexion)
			updatecommand = New OdbcCommand("update ARTICLES set nom=?, prix=?, stockactuel=?, stockminimum=? where id=?", connexion)
			deleteSomeCommand = New OdbcCommand("delete from ARTICLES where id=?", connexion)
			selectSomeCommand = New OdbcCommand("select id, nom, prix, stockactuel, stockminimum from ARTICLES where id=?", connexion)
			updateStockCommand = New OdbcCommand("update ARTICLES set stockactuel=stockactuel+? where id=? and (stockactuel+?)>=0", connexion)
			selectAllCommand = New OdbcCommand("select id, nom, prix, stockactuel, stockminimum from ARTICLES", connexion)
			deleteAllCommand = New OdbcCommand("delete from ARTICLES", connexion)
		End Sub

		Public Function ajouteArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.ajouteArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte d'insertion
				With insertCommand.Parameters
					.Clear()
					.Add(New OdbcParameter("id", unArticle.id))
					.Add(New OdbcParameter("nom", unArticle.nom))
					.Add(New OdbcParameter("prix", unArticle.prix))
					.Add(New OdbcParameter("stockactuel", unArticle.stockactuel))
					.Add(New OdbcParameter("stockminimum", unArticle.stockminimum))
				End With
				Try
					'on l'excute
					Return executeUpdate(insertCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur  l'ajout de l'article [{0}] : {1}", unArticle.ToString, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function changerStockArticle(ByVal idArticle As Integer, ByVal mouvement As Integer) As Integer Implements IArticlesDao.changerStockArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte de mise  jour du stock
				With updateStockCommand.Parameters
					.Clear()
					.Add(New OdbcParameter("mvt1", mouvement))
					.Add(New OdbcParameter("id", idArticle))
					.Add(New OdbcParameter("mvt2", mouvement))
				End With
				'on l'excute
				Try
					Return executeUpdate(updateStockCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors du changement de stock [idArticle={0}, mouvement={1}] : [{2}]", idArticle, mouvement, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Sub clearAllArticles() Implements IArticlesDao.clearAllArticles
			' section exclusive
			SyncLock Me
				Try
					'on excute la requte d'insertion
					executeUpdate(deleteAllCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la suppression des articles : {0}", ex.Message))
				End Try
			End SyncLock
		End Sub

		Public Function getAllArticles() As System.Collections.IList Implements IArticlesDao.getAllArticles
			' section exclusive
			SyncLock Me
				Try
					'on excute la requte select
					Dim articles As IList = executeQuery(selectAllCommand)
					'on retourne le liste
					Return articles
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de l'obtention des articles [select id,nom,prix,stockactuel,stockminimum from articles]: {0}", ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function getArticleById(ByVal idArticle As Integer) As Article Implements IArticlesDao.getArticleById
			' section exclusive
			SyncLock Me
				' on prpare la requte select
				With selectSomeCommand.Parameters
					.Clear()
					.Add(New OdbcParameter("id", idArticle))
				End With
				'on l'excute
				Try
					'on excute la requte
					Dim articles As IList = executeQuery(selectSomeCommand)
					'on test si l'on a trouv l'article
					If articles.Count = 0 Then Return Nothing
					'on retourne l'article
					Return CType(articles.Item(0), Article)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la recherche de l'article [{0} : {1}", idArticle, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function modifieArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.modifieArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte update
				With updatecommand.Parameters
					.Clear()
					.Add(New OdbcParameter("nom", unArticle.nom))
					.Add(New OdbcParameter("prix", unArticle.prix))
					.Add(New OdbcParameter("stockactuel", unArticle.stockactuel))
					.Add(New OdbcParameter("stockminimum", unArticle.stockminimum))
					.Add(New OdbcParameter("id", unArticle.id))
				End With
				' on l'excute
				Try
					'on excute la requte d'insertion
					Return executeUpdate(updatecommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception("Erreur lors de la modification de l'article [" + unArticle.ToString + "]", ex)
				End Try
			End SyncLock
		End Function

		Public Function supprimeArticle(ByVal idArticle As Integer) As Integer Implements IArticlesDao.supprimeArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte delete
				With deleteSomeCommand.Parameters
					.Clear()
					.Add(New OdbcParameter("id", idArticle))
				End With
				'on l'excute
				Try
					'on excute la requte de suppression
					Return executeUpdate(deleteSomeCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la suppression de l'article [id={0}] : {1}", idArticle, ex.Message))
				End Try
			End SyncLock
		End Function

		Private Function executeQuery(ByVal query As OdbcCommand) As IList
			' excution d'une requte SELECT 
			' dclaration de l'objet permettant l'accs  toutes les lignes de la table rsultat
			Dim myReader As OdbcDataReader = Nothing
			Try
				'on cre une connexion  la BDD
				connexion.Open()
				'on excute la requte
				myReader = query.ExecuteReader()
				'on dclare une liste d'articles pour la retourner par la suite
				Dim articles As IList = New ArrayList
				Dim unArticle As Article
				While myReader.Read()
					'on prpare un article avec les valeurs du reader
					unArticle = New Article
					unArticle.id = myReader.GetInt32(0)
					unArticle.nom = myReader.GetString(1)
					unArticle.prix = myReader.GetDouble(2)
					unArticle.stockactuel = myReader.GetInt32(3)
					unArticle.stockminimum = myReader.GetInt32(4)
					'on ajoute l'article  la liste
					articles.Add(unArticle)
				End While
				'on retourne le rsultat
				Return articles
			Finally
				' libration des ressources
				If Not myReader Is Nothing And Not myReader.IsClosed Then myReader.Close()
				If Not connexion Is Nothing Then connexion.Close()
			End Try
		End Function

		Private Function executeUpdate(ByVal sqlCommand As OdbcCommand) As Integer
			' excution d'une requte de mise  jour
			Try
				'on cre une connexion  la BDD
				connexion.Open()
				'on excute la requte
				Return sqlCommand.ExecuteNonQuery()
			Finally
				' libration des ressources
				If Not connexion Is Nothing Then connexion.Close()
			End Try
		End Function
	End Class

End Namespace
