Imports System
Imports System.Collections
Imports System.Data.OleDb

Namespace istia.st.articles.dao

	Public Class ArticlesDaoOleDb
		Implements istia.st.articles.dao.IArticlesDao

		' champs privs
		Private connexion As OleDbConnection = Nothing
		Private insertCommand As OleDbCommand
		Private updatecommand As OleDbCommand
		Private deleteSomeCommand As OleDbCommand
		Private selectSomeCommand As OleDbCommand
		Private updateStockCommand As OleDbCommand
		Private deleteAllCommand As OleDbCommand
		Private selectAllCommand As OleDbCommand

		' constructeur
		Public Sub New(ByVal connectString As String)
			' connectString : chane de connexion  la source OleDb
			'on instancie la connexion
			connexion = New OleDbConnection(connectString)
			' on prpare les requtes SQL
			insertCommand = New OleDbCommand("insert into ARTICLES(id, nom, prix, stockactuel, stockminimum) values (?,?,?,?,?)", connexion)
			updatecommand = New OleDbCommand("update ARTICLES set nom=?, prix=?, stockactuel=?, stockminimum=? where id=?", connexion)
			deleteSomeCommand = New OleDbCommand("delete from ARTICLES where id=?", connexion)
			selectSomeCommand = New OleDbCommand("select id, nom, prix, stockactuel, stockminimum from ARTICLES where id=?", connexion)
			updateStockCommand = New OleDbCommand("update ARTICLES set stockactuel=stockactuel+? where id=? and (stockactuel+?)>=0", connexion)
			selectAllCommand = New OleDbCommand("select id, nom, prix, stockactuel, stockminimum from ARTICLES", connexion)
			deleteAllCommand = New OleDbCommand("delete from ARTICLES", connexion)
		End Sub

		Public Function ajouteArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.ajouteArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte d'insertion
				With insertCommand.Parameters
					.Clear()
					.Add(New OleDbParameter("id", OleDbType.Integer)).Value = unArticle.id
					.Add(New OleDbParameter("nom", OleDbType.VarChar)).Value = unArticle.nom
					.Add(New OleDbParameter("prix", OleDbType.Double)).Value = unArticle.prix
					.Add(New OleDbParameter("stockactuel", OleDbType.Integer)).Value = unArticle.stockactuel
					.Add(New OleDbParameter("stockminimum", OleDbType.Integer)).Value = unArticle.stockminimum
				End With
				Try
					'on l'excute
					Return executeUpdate(insertCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur  l'ajout de l'article [{0}] : {1}", unArticle.ToString, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function changerStockArticle(ByVal idArticle As Integer, ByVal mouvement As Integer) As Integer Implements IArticlesDao.changerStockArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte de mise  jour du stock
				With updateStockCommand.Parameters
					.Clear()
					.Add(New OleDbParameter("mvt1", OleDbType.Integer)).Value = mouvement
					.Add(New OleDbParameter("id", OleDbType.Integer)).Value = idArticle
					.Add(New OleDbParameter("mvt2", OleDbType.Integer)).Value = mouvement
				End With
				'on l'excute
				Try
					Return executeUpdate(updateStockCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors du changement de stock [idArticle={0}, mouvement={1}] : [{2}]", idArticle, mouvement, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Sub clearAllArticles() Implements IArticlesDao.clearAllArticles
			' section exclusive
			SyncLock Me
				Try
					'on excute la requte d'insertion
					executeUpdate(deleteAllCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la suppression des articles : {0}", ex.Message))
				End Try
			End SyncLock
		End Sub

		Public Function getAllArticles() As System.Collections.IList Implements IArticlesDao.getAllArticles
			' section exclusive
			SyncLock Me
				Try
					'on excute la requte select
					Dim articles As IList = executeQuery(selectAllCommand)
					'on retourne le liste
					Return articles
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de l'obtention des articles [select id,nom,prix,stockactuel,stockminimum from articles]: {0}", ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function getArticleById(ByVal idArticle As Integer) As Article Implements IArticlesDao.getArticleById
			' section exclusive
			SyncLock Me
				' on prpare la requte select
				With selectSomeCommand.Parameters
					.Clear()
					.Add(New OleDbParameter("id", OleDbType.Integer)).Value = idArticle
				End With
				'on l'excute
				Try
					'on excute la requte
					Dim articles As IList = executeQuery(selectSomeCommand)
					'on test si l'on a trouv l'article
					If articles.Count = 0 Then Return Nothing
					'on retourne l'article
					Return CType(articles.Item(0), Article)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la recherche de l'article [{0} : {1}", idArticle, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function modifieArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.modifieArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte update
				With updatecommand.Parameters
					.Clear()
					.Add(New OleDbParameter("nom", OleDbType.VarChar)).Value = unArticle.nom
					.Add(New OleDbParameter("prix", OleDbType.Double)).Value = unArticle.prix
					.Add(New OleDbParameter("stockactuel", OleDbType.Integer)).Value = unArticle.stockactuel
					.Add(New OleDbParameter("stockminimum", OleDbType.Integer)).Value = unArticle.stockminimum
					.Add(New OleDbParameter("id", OleDbType.Integer)).Value = unArticle.id
				End With
				' on l'excute
				Try
					'on excute la requte d'insertion
					Return executeUpdate(updatecommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception("Erreur lors de la modification de l'article [" + unArticle.ToString + "]", ex)
				End Try
			End SyncLock
		End Function

		Public Function supprimeArticle(ByVal idArticle As Integer) As Integer Implements IArticlesDao.supprimeArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte delete
				With deleteSomeCommand.Parameters
					.Clear()
					.Add(New OleDbParameter("id", OleDbType.Integer)).Value = idArticle
				End With
				'on l'excute
				Try
					'on excute la requte de suppression
					Return executeUpdate(deleteSomeCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la suppression de l'article [id={0}] : {1}", idArticle, ex.Message))
				End Try
			End SyncLock
		End Function

		Private Function executeQuery(ByVal query As OleDbCommand) As IList
			' excution d'une requte SELECT 
			' dclaration de l'objet permettant l'accs  toutes les lignes de la table rsultat
			Dim myReader As OleDbDataReader = Nothing
			Try
				'on cre une connexion  la BDD
				connexion.Open()
				'on excute la requte
				myReader = query.ExecuteReader()
				'on dclare une liste d'articles pour la retourner par la suite
				Dim articles As IList = New ArrayList
				Dim unArticle As Article
				While myReader.Read()
					'on prpare un article avec les valeurs du reader
					unArticle = New Article
					unArticle.id = myReader.GetInt32(0)
					unArticle.nom = myReader.GetString(1)
					unArticle.prix = myReader.GetDouble(2)
					unArticle.stockactuel = myReader.GetInt32(3)
					unArticle.stockminimum = myReader.GetInt32(4)
					'on ajoute l'article  la liste
					articles.Add(unArticle)
					' suivi
					Console.WriteLine(unArticle.ToString)
				End While
				'on retourne le rsultat
				Return articles
			Finally
				' libration des ressources
				If Not myReader Is Nothing And Not myReader.IsClosed Then myReader.Close()
				If Not connexion Is Nothing Then connexion.Close()
			End Try
		End Function

		Private Function executeUpdate(ByVal sqlCommand As OleDbCommand) As Integer
			' excution d'une requte de mise  jour
			Try
				'on cre une connexion  la BDD
				connexion.Open()
				'on excute la requte
				Return sqlCommand.ExecuteNonQuery()
			Finally
				' libration des ressources
				If Not connexion Is Nothing Then connexion.Close()
			End Try
		End Function
	End Class

End Namespace
