Option Explicit On 
Option Strict On

Imports System
Imports IBatisNet.DataMapper
Imports System.Collections

Namespace istia.st.articles.dao

	Public Class ArticlesDaoSqlMap
		Implements IArticlesDao

		' champs privs
		Dim mappeur As SqlMapper = Mapper.Instance

		' liste de tous les articles
		Public Function getAllArticles() As IList Implements IArticlesDao.getAllArticles
			SyncLock Me
				Try
					Return mappeur.QueryForList("getAllArticles", Nothing)
				Catch ex As Exception
					Throw New Exception("Echec de l'obtention de tous les articles : [" + ex.ToString + "]")
				End Try
			End SyncLock
		End Function

		' ajout d'un article
		Public Function ajouteArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.ajouteArticle
			SyncLock Me
				Try
					' unArticle : article  ajouter
					' insertion
					mappeur.Insert("insertArticle", unArticle)
					Return 1
				Catch ex As Exception
					Throw New Exception("Echec de l'ajout de l'article [" + unArticle.ToString + "] : [" + ex.ToString + "]")
				End Try
			End SyncLock
		End Function

		' supprime un article
		Public Function supprimeArticle(ByVal idArticle As Integer) As Integer Implements IArticlesDao.supprimeArticle
			SyncLock Me
				Try
					' id : id de l'article  supprimer
					' suppression
					Return mappeur.Delete("deleteArticle", idArticle)
				Catch ex As Exception
					If ex.GetType.Equals(GetType(IBatisNet.Common.Exceptions.ConcurrentException)) Then Return 0
					Throw New Exception("Erreur lors de la suppression de l'article d'id [" + idArticle.ToString + "] : [" + ex.ToString + "]")
				End Try
			End SyncLock
		End Function

		' modifie un article
		Public Function modifieArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.modifieArticle
			SyncLock Me
				Try
					' mise  jour
					Return mappeur.Update("modifyArticle", unArticle)
				Catch ex As Exception
					If ex.GetType.Equals(GetType(IBatisNet.Common.Exceptions.ConcurrentException)) Then Return 0
					Throw New Exception("Erreur lors de la mise  jour de l'article [" + unArticle.ToString + "] : [" + ex.ToString + "]")
				End Try
			End SyncLock
		End Function

		' recherche d'un article
		Public Function getArticleById(ByVal idArticle As Integer) As Article Implements IArticlesDao.getArticleById
			SyncLock Me
				Try
					' id : id de l'article recherch
					Return CType(mappeur.QueryForObject("getArticleById", idArticle), Article)
				Catch ex As Exception
					Throw New Exception("Erreur lors de la recherche de l'article d'id [" + idArticle.ToString + "] : [" + ex.ToString + "]")
				End Try
			End SyncLock
		End Function

		' suppression de tous les articles
		Public Sub clearAllArticles() Implements IArticlesDao.clearAllArticles
			SyncLock Me
				Try
					mappeur.Delete("clearAllArticles", Nothing)
				Catch ex As Exception
					Throw New Exception("Erreur lors de l'effacement de la table des articles : [" + ex.ToString + "]")
				End Try
			End SyncLock
		End Sub

		' on change le stock d'un article
		Public Function changerStockArticle(ByVal idArticle As Integer, ByVal mouvement As Integer) As Integer Implements IArticlesDao.changerStockArticle
			SyncLock Me
				Try
					' id : id de l'article dont on change le stock
					' mouvement : mouvement du stock
					Dim paramtres As New Hashtable(2)
					paramtres("id") = idArticle
					paramtres("mouvement") = mouvement
					' mise  jour
					Return mappeur.Update("changerStockArticle", paramtres)
				Catch ex As Exception
					If ex.GetType.Equals(GetType(IBatisNet.Common.Exceptions.ConcurrentException)) Then Return 0
					Throw New Exception(String.Format("Erreur lors du changement de stock [{0},{1}] : {2}", idArticle, mouvement, ex.ToString))
				End Try
			End SyncLock
		End Function
	End Class
End Namespace
