Imports System
Imports System.Collections
Imports FirebirdSql.Data.Firebird

Namespace istia.st.articles.dao

	Public Class ArticlesDaoFirebirdProvider
		Implements istia.st.articles.dao.IArticlesDao

		' champs privs
		Private connexion As FbConnection = Nothing
		Private databasePath As String
		Private insertCommand As FbCommand
		Private updatecommand As FbCommand
		Private deleteSomeCommand As FbCommand
		Private selectSomeCommand As FbCommand
		Private updateStockCommand As FbCommand
		Private deleteAllCommand As FbCommand
		Private selectAllCommand As FbCommand

		' constructeur
		Public Sub New(ByVal serveur As String, ByVal databasePath As String, ByVal uid As String, ByVal password As String)
			' serveur : nom de la machine hte du SGBD Firebird
			' databaseName : chemin d'accs  la base de donnes  exploiter
			' uid : identit de l'utilisateur qui se connecte  la base
			' password : son mot de passe

			'on rcupre le nom de la base pass en argument
			Me.databasePath = databasePath
			'on instancie la connexion
			Dim connectString As String = String.Format("DataSource={0};Database={1};User={2};Password={3}", serveur, databasePath, uid, password)
			connexion = New FbConnection(connectString)
			' on prpare les requtes SQL
			insertCommand = New FbCommand("insert into ARTICLES(id, nom, prix, stockactuel, stockminimum) values (@id,@nom,@prix,@sa,@sm)", connexion)
			updatecommand = New FbCommand("update ARTICLES set nom=@nom, prix=@prix, stockactuel=@sa, stockminimum=@sm where id=@id", connexion)
			deleteSomeCommand = New FbCommand("delete from ARTICLES where id=@id", connexion)
			selectSomeCommand = New FbCommand("select id, nom, prix, stockactuel, stockminimum from ARTICLES where id=@id", connexion)
			updateStockCommand = New FbCommand("update ARTICLES set stockactuel=stockactuel+@mvt where id=@id and (stockactuel+@mvt)>=0", connexion)
			selectAllCommand = New FbCommand("select id, nom, prix, stockactuel, stockminimum from ARTICLES", connexion)
			deleteAllCommand = New FbCommand("delete from ARTICLES", connexion)
		End Sub

		Public Function ajouteArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.ajouteArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte d'insertion
				With insertCommand.Parameters
					.Clear()
					.Add(New FbParameter("@id", unArticle.id))
					.Add(New FbParameter("@nom", unArticle.nom))
					.Add(New FbParameter("@prix", unArticle.prix))
					.Add(New FbParameter("@sa", unArticle.stockactuel))
					.Add(New FbParameter("@sm", unArticle.stockminimum))
				End With
				Try
					'on l'excute
					Return executeUpdate(insertCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur  l'ajout de l'article [{0}] : {1}", unArticle.ToString, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function changerStockArticle(ByVal idArticle As Integer, ByVal mouvement As Integer) As Integer Implements IArticlesDao.changerStockArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte de mise  jour du stock
				With updateStockCommand.Parameters
					.Clear()
					.Add(New FbParameter("@mvt", mouvement))
					.Add(New FbParameter("@id", idArticle))
				End With
				'on l'excute
				Try
					Return executeUpdate(updateStockCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors du changement de stock [idArticle={0}, mouvement={1}] : [{2}]", idArticle, mouvement, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Sub clearAllArticles() Implements IArticlesDao.clearAllArticles
			' section exclusive
			SyncLock Me
				Try
					'on excute la requte d'insertion
					executeUpdate(deleteAllCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la suppression des articles : {0}", ex.Message))
				End Try
			End SyncLock
		End Sub

		Public Function getAllArticles() As System.Collections.IList Implements IArticlesDao.getAllArticles
			' section exclusive
			SyncLock Me
				Try
					'on excute la requte select
					Dim articles As IList = executeQuery(selectAllCommand)
					'on retourne le liste
					Return articles
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de l'obtention des articles [select id,nom,prix,stockactuel,stockminimum from articles]: {0}", ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function getArticleById(ByVal idArticle As Integer) As Article Implements IArticlesDao.getArticleById
			' section exclusive
			SyncLock Me
				' on prpare la requte select
				With selectSomeCommand.Parameters
					.Clear()
					.Add(New FbParameter("@id", idArticle))
				End With
				'on l'excute
				Try
					'on excute la requte
					Dim articles As IList = executeQuery(selectSomeCommand)
					'on test si l'on a trouv l'article
					If articles.Count = 0 Then Return Nothing
					'on retourne l'article
					Return CType(articles.Item(0), Article)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la recherche de l'article [{0} : {1}", idArticle, ex.Message))
				End Try
			End SyncLock
		End Function

		Public Function modifieArticle(ByVal unArticle As Article) As Integer Implements IArticlesDao.modifieArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte update
				With updatecommand.Parameters
					.Clear()
					.Add(New FbParameter("@nom", unArticle.nom))
					.Add(New FbParameter("@prix", unArticle.prix))
					.Add(New FbParameter("@sa", unArticle.stockactuel))
					.Add(New FbParameter("@sm", unArticle.stockminimum))
					.Add(New FbParameter("@id", unArticle.id))
				End With
				' on l'excute
				Try
					'on excute la requte d'insertion
					Return executeUpdate(updatecommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception("Erreur lors de la modification de l'article [" + unArticle.ToString + "]", ex)
				End Try
			End SyncLock
		End Function

		Public Function supprimeArticle(ByVal idArticle As Integer) As Integer Implements IArticlesDao.supprimeArticle
			' section exclusive
			SyncLock Me
				' on prpare la requte delete
				With deleteSomeCommand.Parameters
					.Clear()
					.Add(New FbParameter("@id", idArticle))
				End With
				'on l'excute
				Try
					'on excute la requte de suppression
					Return executeUpdate(deleteSomeCommand)
				Catch ex As Exception
					'erreur de requte
					Throw New Exception(String.Format("Erreur lors de la suppression de l'article [id={0}] : {1}", idArticle, ex.Message))
				End Try
			End SyncLock
		End Function

		Private Function executeQuery(ByVal query As FbCommand) As IList
			' excution d'une requte SELECT 
			' dclaration de l'objet permettant l'accs  toutes les lignes de la table rsultat
			Dim myReader As FbDataReader = Nothing
			Try
				'on cre une connexion  la BDD
				connexion.Open()
				'on excute la requte
				myReader = query.ExecuteReader()
				'on dclare une liste d'articles pour la retourner par la suite
				Dim articles As IList = New ArrayList
				Dim unArticle As Article
				While myReader.Read()
					'on prpare un article avec les valeurs du reader
					unArticle = New Article
					unArticle.id = myReader.GetInt32(0)
					unArticle.nom = myReader.GetString(1)
					unArticle.prix = myReader.GetDouble(2)
					unArticle.stockactuel = myReader.GetInt32(3)
					unArticle.stockminimum = myReader.GetInt32(4)
					'on ajoute l'article  la liste
					articles.Add(unArticle)
				End While
				'on retourne le rsultat
				Return articles
			Finally
				' libration des ressources
				If Not myReader Is Nothing And Not myReader.IsClosed Then myReader.Close()
				If Not connexion Is Nothing Then connexion.Close()
			End Try
		End Function

		Private Function executeUpdate(ByVal updateCommand As FbCommand) As Integer
			' excution d'une requte de mise  jour
			Try
				'on cre une connexion  la BDD
				connexion.Open()
				'on excute la requte
				Return updateCommand.ExecuteNonQuery()
			Finally
				' libration des ressources
				If Not connexion Is Nothing Then connexion.Close()
			End Try
		End Function
	End Class

End Namespace
